# customdelegate.py -- classes for handling input in the cartesian coord table
#
# Copyright (c) Adam Tenderholt, Stanford University, 2005-2006
#                               a-tenderholt@stanford.edu
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

# based on the customdelegate example in the PyQt opensource distribution

import sys
from PyQt4 import QtCore, QtGui
from cclib.parser import utils

table = utils.PeriodicTable()

class ElementValidator(QtGui.QValidator):
    def __init__(self, parent = None):
        QtGui.QValidator.__init__(self, parent)
        self.regexp = QtCore.QRegExp("[A-Z]")

    def validate(self, inputStr, pos):
        if inputStr.length() > 2:
            return (QtGui.QValidator.Invalid,pos)

        if inputStr.length() == 0:
            return (QtGui.QValidator.Intermediate, pos)

        if not self.regexp.exactMatch(inputStr[0]):
            return (QtGui.QValidator.Invalid,pos)

        try:
            table.number[str(inputStr)]
        except KeyError:
            return (QtGui.QValidator.Invalid,pos)
        else:
            return (QtGui.QValidator.Acceptable,pos)

class CustomDelegate(QtGui.QItemDelegate):
    def __init__(self, parent = None):
        QtGui.QItemDelegate.__init__(self, parent)

    def createEditor(self, parent, option, index):
        editor = QtGui.QLineEdit(parent)

        if index.column() == 0:
            validator = ElementValidator(editor)
        else:
            validator = QtGui.QDoubleValidator(-1000,1000,5,editor)
        editor.setValidator(validator)
            
        editor.installEventFilter(self)

        return editor

    def setEditorData(self, lineEdit, index):
        value = index.model().data(index, QtCore.Qt.DisplayRole).toString()
        lineEdit.setText(value)

    def setModelData(self, lineEdit, model, index):
        value = lineEdit.text()
        if value == "":
            return

        if index.column() > 0:
            value = "%7.3f" % float(value)

        model.setData(index, QtCore.QVariant(value), QtCore.Qt.DisplayRole)

    def updateEditorGeometry(self, editor, option, index):
        editor.setGeometry(option.rect)


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)

    model = QtGui.QStandardItemModel(4, 2)
    tableView = QtGui.QTableView()
    tableView.setModel(model)

    delegate = CustomDelegate()
    tableView.setItemDelegate(delegate)

    for row in range(4):
        for column in range(2):
            index = model.index(row, column, QtCore.QModelIndex())
            model.setData(index, QtCore.QVariant((row+1) * (column+1)))

    tableView.setWindowTitle("Spin Box Delegate")
    tableView.show()
    sys.exit(app.exec_())
