# geomwidget.py -- widget used in the geometry optimization tab
#
# Copyright (c) Adam Tenderholt, Stanford University, 2005-2006
#                               a-tenderholt@stanford.edu
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

import sys
from os.path import exists

from PyQt4 import QtGui,QtCore
from cclib.parser import ccopen
from cclib.bridge import makeopenbabel
import openbabel

from glwidget import *

class GeomWidget(QtGui.QWidget):
    
    def __init__(self, parent = None, flags = 0):
        QtGui.QWidget.__init__(self)

        self.setObjectName("geomWidget")

#setup widgets
        self.gridlayout = QtGui.QGridLayout(self)
        self.gridlayout.setMargin(9)
        self.gridlayout.setSpacing(6)
        self.gridlayout.setObjectName("gridlayout")

        self.glWidget = GLWidget(self)
        self.glWidget.setObjectName("glWidget")
        self.gridlayout.addWidget(self.glWidget,0,0,1,2)
        self.glWidget.show()

        self.spinBox = QtGui.QSpinBox(self)
        self.spinBox.setObjectName("spinBox")
        self.spinBox.setMinimum(0)
        self.spinBox.setMaximum(0)
        self.gridlayout.addWidget(self.spinBox,1,0,1,1)

        self.slider = QtGui.QSlider(self)
        self.slider.setOrientation(QtCore.Qt.Horizontal)
        self.slider.setObjectName("slider")
        self.slider.setMinimum(0)
        self.slider.setMaximum(0)
        self.gridlayout.addWidget(self.slider,1,1,1,1)

#slots
        QtCore.QObject.connect(self.spinBox,QtCore.SIGNAL("valueChanged(int)"),self.slotUpdateGeometry)
        QtCore.QObject.connect(self.slider,QtCore.SIGNAL("valueChanged(int)"),self.slotUpdateGeometry)
       
#class variables
        self.data = None

    def setData(self,data):
        self.data = data

        if not hasattr(data, "atomcoords"):
            return

        if not hasattr(data, "atomnos"):
            return

        size = len(data.atomcoords)
        self.spinBox.setMaximum(size - 1)
        self.slider.setMaximum(size - 1)
        
        self.glWidget.setMolecule(data.atomcoords[0], data.atomnos)

    def slotUpdateGeometry(self, value):

        if not self.data:
            return

        self.glWidget.setMolecule(self.data.atomcoords[value], self.data.atomnos)
        self.slider.setValue(value)
        self.spinBox.setValue(value)

        self.emit(QtCore.SIGNAL("signalUpdateGeometry(int)"),value)

if __name__=='__main__':
    app = QtGui.QApplication(sys.argv)
    window = GeomWidget()
    window.resize(700,500)
    window.show()
    if len(sys.argv)==2 and exists(sys.argv[1]):
        parser = ccopen(sys.argv[1])
        data = parser.parse()
        window.setData(data) 

    sys.exit(app.exec_())

